/*
 * created by: Timo Lindhorst <timo.lindhorst@student.uni-magdeburg.de>
 */

import josx.platform.rcx.*;
import java.lang.Math;
import java.lang.Thread;

/**
 * This class implements a simple controller for the RCX.
 */
public class Controller extends Thread {
/*
 * How the controller works.
 * The grade of the curve is understood as a quotient: (rotation left / rotation right).
 * The actual quotient is compared to the wanted. The motor on that side which is 
 * too fast, is switched off as long as the quotient reaches the wanted level.
 */
 
  int sampleRate;
  Motor[] motors;
  Sensor[] sensors;
  int[] sensorValues;
  int speed;
  float wanted;
  float current;
  float diff;
    
  public Controller() {
    sampleRate = 1;
    motors = new Motor[2];
    sensors = new Sensor[2];
    sensorValues = new int[2];
    wanted = 1;
    current = 1;
    speed = 0;
  
    motors[0] = Motor.A;
    motors[1] = Motor.C;
    sensors[0] = Sensor.S1;
    sensors[1] = Sensor.S3;
    
    for(int i=0; i<sensors.length; i++) {
      sensors[i].setTypeAndMode(SensorConstants.SENSOR_TYPE_ROT,
                                SensorConstants.SENSOR_MODE_ANGLE);
      sensors[i].setPreviousValue(0);
      sensors[i].activate();
    }
  }
  
  /** 
   * Set new speed.
   *
   * @param speed Speed level from -3 to +3. A positive value meens forward, a negative backward movement.
   */
  public void setSpeed(int speed) {
    if (Math.abs(speed) == 0) { this.speed = 0; }
    else if (Math.abs(speed) == 1) { this.speed = 1; }
    else if (Math.abs(speed) == 2) { this.speed = 4; }
    else if (Math.abs(speed) == 3) { this.speed = 7; }
    else { this.speed = 7; }
    
    if (speed == 0) {
      motors[0].stop();
      motors[1].stop();
    } else if (speed < 0) { 
      this.speed *= -1;
      motors[0].backward();
      motors[1].backward();
    } else {
      motors[0].forward();
      motors[1].forward();
    }
    motors[0].setPower(Math.abs(speed));
    motors[1].setPower(Math.abs(speed));

    sensors[0].setPreviousValue(0);
    sensors[1].setPreviousValue(0);
  }
  
  /**
   * Set the direction to drive a curve.
   *
   * @param grade The grade of the curve [-15;15]. A negative value causes a curve to the left, a positive to the right. A high value causes a lower diameter.
   */
  public void setDirection(int grade) {
    sensors[0].setPreviousValue(0);
    sensors[1].setPreviousValue(0);
    wanted = 1 + 0.05f * Math.abs(grade);     // 1.0 to 1.8
    if (grade < 0) { wanted = 1 / wanted; }
  }
  
  /**
   * Updates diff.
   */
  void updateValues() {
    sensorValues[0] = sensors[0].readValue();
    sensorValues[1] = sensors[1].readValue();
    try {
      current = (float) sensorValues[0] / (float) sensorValues[1];
    } catch (Exception e) { // ArithmeticExeption div by zero
      current = wanted; //1.0f;
    }
    diff = current - wanted;
  }
  
  public void run() {
    Motor motor;
    float oldDiff;
    
    while(!isInterrupted()) {
      updateValues();
    
      if (diff<0) {
        motor = motors[1];
      } else if (diff>0) {
        motor = motors[0];
      } else {
        continue;
      }
      
//      motor.flt();
      motor.setPower(0);
      
      do {
        oldDiff = diff;
        updateValues();
      } while ((diff * oldDiff) > 0);    // until sign changes
      
//       motor.forward();
      motor.setPower(abs(speed));
      
      LCD.showNumber(abs(speed));
      Thread.yield();

    }
    motors[0].stop();
    motors[1].stop();
  }
  
  int abs(int n) {
    return((n<0) ? -n : n);
  }
    
  public static void main(String[] args) throws InterruptedException {
    /* For testing purpose */
    
    Controller contr = new Controller();

    contr.setSpeed(3);
    contr.setDirection(0);
    contr.start();
    LCD.showNumber(0);
    Button.RUN.waitForPressAndRelease();
    LCD.showNumber(4);
    contr.setDirection(-4);
    Button.RUN.waitForPressAndRelease();
    LCD.showNumber(8);
    contr.setDirection(8);
    Button.RUN.waitForPressAndRelease();
    LCD.showNumber(12);
    contr.setDirection(-12);
    Button.RUN.waitForPressAndRelease();
    LCD.showNumber(15);
    contr.setDirection(15);
    Button.RUN.waitForPressAndRelease();
    contr.interrupt();
  }

}
