/* External error codes */
#define RCX_OK                  (   0)
#define RCX_E_PROGRAM_FAILURE   (-100)
#define RCX_E_DEVICE_NOT_FOUND  (-101)
#define RCX_E_DEVICE_READONLY   (-102)
#define RCX_E_DEVICE_NO_LIRC    (-103)
#define RCX_E_DEVICE_NOT_OPEN   (-104)
#define RCX_E_DEVICE_IS_OPEN    (-105)
#define RCX_E_DEVICE_ERROR      (-106)
#define RCX_E_RECV_NOTHING      (-107)
#define RCX_E_RECV_ERROR        (-108)

// motors for setMotorDirection(), setMotorState() and setMotorPower()
#define MOTOR_A 0x01
#define MOTOR_B 0x02
#define MOTOR_C 0x04

// sensors
#define SENSOR_1 0
#define SENSOR_2 1
#define SENSOR_3 2

// sensortypes
#define RAW_SENSOR 0
#define TOUCH 1
#define TEMPERATURE 2
#define LIGHT 3
#define ROTATION 4

// sensormodes
#define RAW_MODE 0
#define BOOLEAN_MODE 1
#define EDGE_COUNT 2
#define PULSE_COUNT 3
#define PERCENTAGE 4
#define CELSIUS 5
#define FAHRENHEIT 6
#define ANGLE 7

// direct for setMotorDirection()
#define MOTOR_FORWARD 0x80
#define MOTOR_REVERSE 0x00
#define MOTOR_FLIP_DIRECTION 0x40

// state for setMotorState()
#define MOTOR_ON 0x80
#define MOTOR_OFF 0x40
#define MOTOR_FLOAT 0x00

// sound for playSound()
#define BEEP 0
#define BEEPBEEP 1
#define DOWN_TONES 2
#define UP_TONES 3
#define BUZZ 4
#define FAST_UP_TONES 5

// transmitter ranges
#define SHORT_RANGE 0
#define LONG_RANGE 1



// one-byte datatype for RCX
typedef char byte;



/* initiate a communication with the RCX
 *
 * init_comm() must be called before any command could be sent.
 * Returns external error codes defined in the headerfile.
 */
int init_comm(void);


/* close the Lirc driver
 *
 * Returns RCX_OK
*/
int close_comm(void);


/******* NOTE *************************************************************
 *                                                                        *
 * If nothing else is specified, the return value of a function is        *
 *   - 1	command excepted by the RCX                               *
 *   - 0	transmission error                                        *
 **************************************************************************/


/* test, whether the RCX is alive
 *
 * return	1		RCX alive
 *		0		RCX not alive
 */
int rcxAlive(void);


/* Request the battery voltage from the RCX
 *
 * Input :	none
 * Output:	voltage		battery power in mV
 */
int getBatteryPower(unsigned short *voltage);


/* Set the power down delay of the RCX to the specified value,
 * measured in minutes. If the power down delay is set to 0, the 
 * RCX remains on indefinitely and the powerOff() command is
 * ignored.
 */
int setPowerDownDelay(byte min);


/* Turn off the RCX unless the power down delay is not zero */
int powerOff(void);


/* Set the value of the message buffer to msg.
 * The RCX gives no matching reply to this command.
 */
void setMessage(byte msg);


/* return the value of messege buffer 
 * 
 * Input:	none
 * Output:	msg		value of the RCX's message buffer
 */
int getMessage(byte *msg);


/* set the transmitter to a short or a long range
 * 
 * Input:	range: SHORT_RANGE, LONG_RANGE
 */
int setTransmitterRange(byte range);


/* Play a sound 
 *
 *   Input	sound:
 *		  BEEP, BEEPBEEP, DOWN_TONES, UP_TONES, BUZZ or FAST_UP_TONES
 */
int playSound(byte sound);


/* play a tone 
 * 
 * Input:  	freq 		frequency, measured in Hz 
 * 		duration	duration, measured in 1/100ths of a second
 */
int playTone(unsigned short freq, byte duration);


/* set the direction for one or more motors
 *
 * Input:	motor:	 MOTOR_A, MOTOR_B, MOTOR_C
 *			   ('MOTOR_A | MOTOR_B' for both MOTOR_A and MOTOR_B)
 *		direction: MOTOR_FORWARD, MOTOR_REVERSE, MOTOR_FLIP_DIRECTION	
 */
int setMotorDirection(byte motor, byte direct);


/* get a motor direction
 *
 * Input:	motor:  MOTOR_A, MOTOR_B, MOTOR_C
 * Output:	direct: MOTOR_FORWARD, MOTOR_REVERSE
 */
int getMotorDirection(byte motor, byte *direct);


/* set the state for one or more motors
 *
 * Input:	motor:	 MOTOR_A, MOTOR_B, MOTOR_C
 *			   ('MOTOR_A | MOTOR_B' for both MOTOR_A and MOTOR_B)
 *		state: MOTOR_ON, MOTOR_OFF, MOTOR_FLOAT	
 */
int setMotorState(byte motor, byte state);


/* get a motor state
 *
 * Input:	motor:  MOTOR_A, MOTOR_B, MOTOR_C
 * Output:	state: MOTOR_ON, MOTOR_OFF, MOTOR_FLOAT
 */
int getMotorState(unsigned char motor, char *state);


/* set the power for one or more motors
 *
 * Input:	motor:	 MOTOR_A, MOTOR_B, MOTOR_C
 *			   ('MOTOR_A | MOTOR_B' for both MOTOR_A and MOTOR_B)
 *		power: a value in 0 .. 7
 */
int setMotorPower(byte motor, byte level);


 /* get the power off a motor
 *
 * Input:	motor:  MOTOR_A, MOTOR_B, MOTOR_C
 * Output:	power: a value in 0 .. 7
 */
int getMotorPower(byte motor, byte *power);


/* Set the type of a sensor
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 *		type: RAW_SENSOR, TOUCH, TEMPERATURE, LIGHT, ROTATION
 */
int setSensorType(byte sensor, byte type);


/* get a sensor type
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 * Output:	type: RAW_SENSOR, TOUCH, TEMPERATURE, LIGHT, ROTATION
 */
int getSensorType(byte sensor, byte *type);


/* Set a sensors mode and slope.
 * 
 * Input:   sensor: SENSOR_1, SENSOR_2, SENSOR_3 
 * 	    mode:
 *		RAW_MODE	Value in 0..1023
 *		BOOLEAN_MODE	Either 0 or 1
 *		EDGE_COUNT	Number of boolean transitions
 *		PULSE_COUNT	Number of boolean transitions divided by two
 *		PERCENTAGE	Raw value scaled to 0..100
 *		CELSIUS		1/10ths of a degree Celsius, -19.8..69.5
 *		FAHRENHEIT	1/10ths of a degree Fahrenheit, -3.6..157.1
 *		ANGLE		1/16ths of a rotation
 *	    
 *	   slope: a value in 0 .. 31 (see description below)
 *
 * The slope value controls 0/1 detection for the BOOLEAN_MODE, EDGE_COUNT
 * and PULSE_COUNT modes. A slope of 0 causes raw sensor values greater than
 * 562 to cause a transition to 0 and raw sensor values less than 460 to cause
 * a transition to 1. The hysteresis prevents bouncing between 0 and 1 near
 * the transition point. A slope value in 1..31, inclusive, causes a
 * transition to 0 or to 1 whenever the difference between consecutive raw
 * sensor values exceeds the slope. Increases larger than the slope result
 * in 0 transitions, while decreases larger than the slope result 
 * in 1 transitions. Note the inversions: high raw values correspond to a
 * boolean 0, while low raw values correspond to a boolean 1.
 */
int setSensorMode(byte sensor, byte mode, byte slope);


 /* get a sensor mode
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 * Output:	mode: RAW_MODE, BOOLEAN_MODE, EDGE_COUNT, PULSE_COUNT, 
 * 		  PERCENTAGE, CELSIUS, FAHRENHEIT, ANGLE
 */
int getSensorMode(byte sensor, byte *mode);


/* get a sensor slope
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 * Output:	slope: a value in 0..31 (for a description see setSensorMode())
 */
int getSensorSlope(byte sensor, byte *slope);


/* get a sensor value
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 * Output:	val: the sensors value in the current mode
 */
int getSensorValue(byte sensor, short *val);


/* get the raw value of a sensor, irrespective of the current sensor mode
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 * Output:	val: the sensors value in raw mode ( 0 .. 1023 )
 */
int getRawSensorValue(byte sensor, short *val);


/* get the boolean value of a sensor, irrespective of the current sensor mode
 *
 * Input:	sensor: SENSOR_1, SENSOR_2, SENSOR_3
 * Output:	val: the sensors value as a boolean
 */
int getBooleanSensorValue(byte sensor, short *val);
