/*
 * remotecontrol.c
 *
 * created by: Timo Lindhorst <timo.lindhorst@student.uni-magdeburg.de>
 *
 * Read instructions from a connected joystick and send control-information to
 * the roboconrol-program via udp-broadcast.
 */
 
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <linux/joystick.h>

#define JS_DEV "/dev/input/js0"
#define NAME_LENGTH 128
#define MAX_VAL 32767
#define MAX_POW 8
#define MAX_SPEED 3
#define MAX_DIRECTION 15
#define MIN_SEND_DELAY 5500


static void update_parameters(int *dpeed, int *direction, int forward, int backward, int wheel);
static int send_data(int socket, int speed, int direction);


int const packetSize = 1400;
unsigned short threshold = 75;     // percent of packets the robot must receive
unsigned int timeout = 500;        // timeout for robot when rate is lower than threshold

int main (int argc, char **argv) {
	int fd; 
	char name[NAME_LENGTH];
	struct js_event jsevent;
	int speed, direction;
	int forward, backward, wheel;
    int lastSpeed;   
	fd_set rfds;
	struct timeval tv;
	int udpSocket;
	struct sockaddr_in receiverAddr;
    int broadcast = 1;   
    int port = 55555;
    int sendDelay = MIN_SEND_DELAY;
    int lock = 1;
    
	
    extern char *optarg;
    
    struct option long_options[] = {
        { "port", required_argument, 0, 'p' },
        { "delay", required_argument, 0, 'd' },
        //{ "size", required_argument, 0, 's' },
        { "threshold", required_argument, 0, 't' },
        { "timeout", required_argument, 0, 'o' },
        { "help", no_argument, 0, 'h' }
    };
    
    int option;
    
    while((option = getopt_long(argc, argv, "p:d:s:t:o:h", long_options, 0)) != -1) {
        switch(option) {
            case 'p':
                port = atoi(optarg);
                break;
            case 'd':
                sendDelay = atoi(optarg);
                break;
//             case 's':
//                 packetSize = atoi(optarg);
//                 break;
            case 't':
                threshold = atoi(optarg);
                if (threshold > 100)
                    threshold = 100;
                break;
            case 'o':
                timeout = atoi(optarg);
                break;
            case 'h':
            default:
                printf("use: remotecontrol [option]\n\n");
                printf("Options: \n");
                printf("\t-p, --port=PORT          send to port PORT\n");
                printf("\t-d, --delay=DELAY        wait DELAY usecs before sending next packet\n");
//                 printf("\t-s, --size=SIZE          send packets of SIZE byte\n");
                printf("\t-t, --threshold=PERCENT  robot must receive PERCENT%% of the packets\n");
                printf("\t-o, --timeout=TIMEOUT    robot waits TIMEOUT ms in case of failure\n");
                printf("\t-h, --help               show this help\n");
                printf("\nSome values could be modified at runtime while pressing Z or C:\n");
                printf("X/A \t\t\t- decrease/increase threshold\n");
                printf("Y/B \t\t\t- decrease/increase timeout\n");
                printf("gear change left/right \t- increase/decrease delay\n\n");
                exit(1);
        }
    }
    
    printf("remotecontrol - port %i, ", port);
    printf("send delay: %.2f ms, packet size: %i bytes\n", (double)sendDelay/1000, packetSize);
    printf("values for robot - threshold: %i%%, timeout:%ims\n", threshold, timeout);
	
	if ((fd = open(JS_DEV, O_RDONLY)) < 0) {
		perror("jscontrol: open joystick device failed");
		exit(1);
	}
	
	ioctl(fd, JSIOCGNAME(NAME_LENGTH), name);
	
	printf("'%s' found!\n", name);
	if (strcmp(name, "Microsoft SideWinder Force Feedback Wheel (USB)") != 0) {
		perror("jscontrol: 'Microsoft SideWinder Force Feedback Wheel (USB)' needed!");
		exit(1);
	}
	
	udpSocket = socket(PF_INET, SOCK_DGRAM, 0);
	if (udpSocket < 0) {
		perror("error opening socket!");
		exit(1);
	}
	
	if(setsockopt(udpSocket, SOL_SOCKET, SO_BROADCAST, &broadcast, sizeof(broadcast)) != 0) {
		perror("setsockopt() failed!");
		exit(1);
	}
	
	receiverAddr.sin_family = AF_INET;
	receiverAddr.sin_port   = htons(port);
	if (inet_aton("255.255.255.255", &receiverAddr.sin_addr) == 0) {
		perror("address not valid");
		exit(1);
	}
	
	connect(udpSocket, (struct sockaddr*) &receiverAddr, sizeof(receiverAddr));
	
	speed = lastSpeed = direction = 0;
	forward = backward = wheel = 0;
        tv.tv_sec = 0;
        tv.tv_usec = sendDelay;
		
	while (1) {  
          
	  FD_ZERO(&rfds);
      FD_SET(fd, &rfds);
	
      // send a packet every sendDelay usecs
	  if(select(fd+1, &rfds, NULL, NULL, &tv) == 0) {
        if ( ! ((lastSpeed == 0) && (speed == 0))) {    // no action -> no transfer
            send_data(udpSocket, speed, direction);
        }
        lastSpeed = speed;
	    tv.tv_usec = sendDelay;
      } else if (FD_ISSET(fd, &rfds)) {
	    if (read(fd, &jsevent, sizeof(struct js_event)) != sizeof(struct js_event)){
              perror("\njscontrol: error reading");
              exit (1);
        }
	
	    switch (jsevent.type & ~JS_EVENT_INIT) {
            case JS_EVENT_AXIS:   
                switch (jsevent.number) {
                    case 1:	// right pedal: forward
                        forward = (-jsevent.value + MAX_VAL) / 2;	//-MAX_VAL to +MAX_VAL
                        break;
                    case 5:	// left pedal: backward
                        backward = (-jsevent.value + MAX_VAL) /2;
                        break;
                    case 0:	// wheel
                        wheel = (jsevent.value);
                        break;
                }
                break;
            
            case JS_EVENT_BUTTON:
                if ((jsevent.number == 2) || (jsevent.number == 5)) {
                    lock = ! jsevent.value;
                }
                
                // change of parameters not allowed
                if (lock) break;
                    
                switch (jsevent.number) {
                    case 6:     // increase sendDelay
                        if (jsevent.value == 1) {
                            sendDelay += 100;
                            printf("increased send delay to %i ms\n", sendDelay);
                        }
                        break;
                    case 7:     // decrease sendDelay
                        if (jsevent.value == 1) {
                            if((sendDelay - 100) >= MIN_SEND_DELAY) {
                                sendDelay -= 100;
                                printf("decreased send delay to %i ms\n", sendDelay);
                            } else {
                                printf("already at minimum send delay!\n");
                            }
                        }
                        break;
                    case 3:     // decrease threshold
                        if (jsevent.value == 1) {
                            if((threshold - 5) >= 0) {
                                threshold -= 5;
                                printf("decreased threshold to %i%%\n", threshold);
                            } else {
                                printf("already at minimum threshold!\n");
                            }
                        }
                        break;
                    case 0:     // increase threshold
                        if (jsevent.value == 1) {
                            if((threshold +5 ) <= 100) {
                                threshold += 5;
                                printf("increased threshold to %i%%\n", threshold);
                            } else {
                                printf("already at maximum threshold!\n");
                            }
                        }
                        break;
                    case 4:     // decrease timout
                        if (jsevent.value == 1) {
                            if((timeout - 100) >= 100) {
                                timeout -= 100;
                                printf("decreased timeout to %i ms\n", timeout);
                            } else {
                                printf("already at minimum timeout!\n");
                            }
                        }
                        break;
                    case 1:     // increase timeout
                        if (jsevent.value == 1) {
                            timeout += 100;
                            printf("increased timeout to %i ms\n", timeout);
                        }
                        break;
                }
                break;
        }
                      
        update_parameters(&speed, &direction, forward, backward, wheel);
        }
	}
	
	return(0);
}

static int round(double x) {
    if (x>=0){ return((int) x + 0.5); }
    else { return((int) x - 0.5); }
}

/*
 * calculates the values for speed and direction from forward, backward and wheel
 */
static void update_parameters(int *speed, int *direction, int forward, int backward, int wheel) {
    
    *speed = forward - backward;
    *speed = round((((double) *speed)/ MAX_VAL) * MAX_SPEED);
    
    *direction = round((((double) wheel) / ((double) MAX_VAL)) * (double) MAX_DIRECTION);
}


/*
 * send the current values speed, direction, threshold and timeout to socket
 */
static int send_data(int socket, int speed, int direction) {
	static int buffer[packetSize / sizeof(int)];
    static int serialnr = 1;
		
	buffer[0] = speed;
	buffer[1] = direction;
    buffer[2] = serialnr++;
    buffer[3] = threshold;
    buffer[4] = timeout;
    
	return(send(socket, &buffer, packetSize, 0));
}
