/*
 * robocontrol.c
 *
 * created by: Timo Lindhorst <timo.lindhorst@student.uni-magdeburg.de>
 */
 
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "rcxcomm.h"

#define RECV_BUFFER_SIZE 1500
#define USEC_TO_WAIT 100000


struct confState {
	int speed;
	int direction;
	unsigned int oldSerialNr;
	unsigned int packetsReceived;
    unsigned int packetsLost;
    unsigned short threshold;
    unsigned int timeout;
};

static int read_packet(int udpSocket, int *buffer);
static int set_power(struct confState *conf);
static void update_conf(int *buffer, struct confState *conf);

int main (int argc, char **argv) {
	int udpSocket;
    struct sockaddr_in receiverAddr;
    int port;
	int buffer[RECV_BUFFER_SIZE/sizeof(int)];
	int broadcast;
	fd_set rfds;
	struct timeval tv;
    
    if (argc > 2) {
        printf("use: robocontrol [<port>]\n"
               "     default port is 55555\n");
        exit(1);
    } else if (argc == 2) 
        port = atoi(argv[1]);
    else {
        port = 55555;
	printf("to change default port use: robocontrol <port>\n");
    }
        
    printf("robocontrol - listening on port %i\n", port);
        
    struct confState currentConfState = {
        speed           : 0,
        direction       : 0,
        oldSerialNr     : 0,
        packetsReceived : 0,
        packetsLost     : 0,
        threshold       : 75,
        timeout         : 500
    };
    
	broadcast = 1;
	
	udpSocket = socket(PF_INET, SOCK_DGRAM, 0);
	if (udpSocket < 0) {
		perror("error opening socket!");
		exit(1);
	}
	
	setsockopt(udpSocket, SOL_SOCKET, SO_BROADCAST, &broadcast, sizeof(broadcast));
	
	receiverAddr.sin_family = AF_INET;
	receiverAddr.sin_port   = htons(port);
	receiverAddr.sin_addr.s_addr = INADDR_ANY;
		
	bind(udpSocket, (struct sockaddr*) &receiverAddr, sizeof(receiverAddr));
	
	init_comm();
        
        tv.tv_sec = 0;
        tv.tv_usec = USEC_TO_WAIT;
        
	
	while(1) {
                    
          FD_ZERO(&rfds);
          FD_SET(udpSocket, &rfds);
                          
          if(select(udpSocket + 1, &rfds, NULL, NULL, &tv) == 0) {
            if (set_power(&currentConfState)) {     // rate was ok
              tv.tv_usec = USEC_TO_WAIT;
            } else {                                // rate was beneath the threshold
              tv.tv_usec = currentConfState.timeout * 1000;
              system("cat failure.au > /dev/dsp");
            }
            currentConfState.packetsReceived = 0;
            currentConfState.packetsLost     = 0;
          } else if (FD_ISSET(udpSocket, &rfds)) {
            if(read_packet(udpSocket, buffer) < 0) {
              printf("read failed!\n");
            } else {
              update_conf(buffer, &currentConfState);   
            }
          }
	}
	
	close_comm();
	close(udpSocket);
	return(0);
}

/*
 * read packet from socket
 * returns success
 */
static int read_packet(int udpSocket, int *buffer) {
        //printf("read_packet\n");
	return(recv(udpSocket, buffer, RECV_BUFFER_SIZE, 0));
}

/*
 * send the current data to the RCX and return success
 */
static int set_power(struct confState *conf) {
    double rate;                // the rate of packets received
    unsigned char byteCode;     // code to send to the iPAQ
    int speed;                  // speed might be changed
    
    if (conf->packetsReceived == 0) {
        rate = 1;
    } else {
        rate = ((double) conf->packetsReceived) / 
                (conf->packetsReceived + conf->packetsLost);
    }
    
    if ((rate*100) >= conf->threshold) {
        speed = conf->speed;
    } else {
        speed = 0;
    }
    
    printf("rt:%5.1f%%", rate*100);      
    printf(", pk:%2u", conf->packetsReceived);
    printf(", lst:%2u", conf->packetsLost);
    printf(", sp:%1i dir:%1i",speed, conf->direction);
    printf("\n");
    
    /*
     * The bytecode is build as following:
     *   bit 0-3: abs(direction)
     *   bit   4: sign(direction)
     *   bit 5-6: abs(speed)
     *   bit   7: sigh(speed)
     */
    byteCode = ((unsigned char) abs(conf->direction)) & 0x0f;
    if (conf->direction < 0) { byteCode |= 0x10; }
    byteCode |= ((((unsigned char) abs(speed)) << 5) & 0x60);
    if (speed < 0) { byteCode |= 0x80; }
    
    setMessage((byte) byteCode);
        
    return((rate*100) >= conf->threshold);
}

/*
 * read new values from buffer and update the confState
 */
static void update_conf(int *buffer, struct confState *conf) {
    conf->speed  = buffer[0];
    conf->direction = buffer[1];
    conf->packetsReceived++;
    
    conf->packetsLost += (buffer[2] - conf->oldSerialNr) - 1;
//     printf(", lst: %2u\n", conf->packetsLost);
    conf->oldSerialNr = buffer[2];
    
    conf->threshold = buffer[3];
    conf->timeout = buffer[4];
}
